// toShare rest.js
'use strict';
// inoutboard

const restVer = '/v0';
const basePath = location.origin + restVer;

function formatDate(date, format){
	const week = ['日','月','火','水','木','金','土'];
	format = format.replace(/yyyy/g, date.getFullYear());
	format = format.replace(/MM/g, ('0' + (date.getMonth() + 1)).slice(-2));
	format = format.replace(/dd/g, ('0' + date.getDate()).slice(-2));
	format = format.replace(/M/g, ('' + (date.getMonth() + 1)).slice(-2));
	format = format.replace(/d/g, ('' + date.getDate()).slice(-2));
	format = format.replace(/HH/g, ('0' + date.getHours()).slice(-2));
	format = format.replace(/mm/g, ('0' + date.getMinutes()).slice(-2));
	format = format.replace(/ss/g, ('0' + date.getSeconds()).slice(-2));
	format = format.replace(/SSS/g, ('00' + date.getMilliseconds()).slice(-3));
	format = format.replace(/week/g, week[date.getDay()]);
	return format;
};

function altHCO(target, defaultValue){
	// 2020 Nullish coalescing operator ??
	return ((target != null)||(target != undefined)) ? target : defaultValue;
}
class restClient {
	constructor(url, jsonRoot){
		if(!url){throw Error(`${this.constructor.name}.restClient: url is a required argument.`);}
		this.jsonRoot = jsonRoot ? jsonRoot : '';
		this.url = url;
		this.baseURL = this.url;
		this.data = {};
	}
	async restFetch(initOpt){
		const response = await fetch(this.url,initOpt);
		if (!response.ok){
			throw Error(`${this.constructor.name}.restFetch: Error response: ${response.status} ${response.statusText}`);
		}
		return response;
	}
	async restGet(){
		const response = await this.restFetch({
			method: 'GET',
			headers: {'Accept': 'application/json'}
		});
		const contentType = await response.headers.get('content-type');
		if (!contentType || !contentType.includes('application/json')){
			throw new TypeError(`${this.constructor.name}.restGet: mime not json`);
		}
		const json = await response.json();
		if(!json){throw Error(`${this.constructor.name}.restGet: json schema error`);}
		this.data = json[this.jsonRoot];
		return response;
	}
	async restPost(jsondata){
		if(!jsondata){throw Error(`${this.constructor.name}.restPost: unknown json`);}
		const response = await this.restFetch({
			method: 'POST',
			headers: {'Content-Type': 'application/json'},
			body: jsondata
		});
		return response;
	}
	async restPut(jsondata){
		if(!jsondata){throw Error(`${this.constructor.name}.restPut: unknown json`);}
		const response = await this.restFetch({
			method: 'PUT',
			headers: {'Content-Type': 'application/json'},
			body: jsondata
		});
		return response;
	}
	async restDelete(){
		const response = await this.restFetch({
			method: 'DELETE'
		});
		return response
	}
	findField(fieldName, value){
		try{
			return this.data.find(d => d[fieldName] === value);
		}catch(e){
			return undefined;
		}
	}
}

class restConfiguration extends restClient {
	constructor(){
		super(`${basePath}/configuration`, 'config');

		this.onUserNameClick = this.onUserNameClick.bind(this)
		this.loadedInoutEditLayer = false;
		this.linkColorCss = document.createElement('link');

		this.linkColorCss.rel = 'stylesheet';
		this.linkColorCss.type = 'text/css';
		document.head.appendChild(this.linkColorCss);
	}
	restGet(items){
		const query = new URLSearchParams({'items': items.join(",")});
		this.url = `${this.baseURL}?${query}`;
		return super.restGet();
	}
	restPut(item, value){
		const jsondata = JSON.stringify({
			[this.jsonRoot]:[{
				'item': item,
				'value': value
			}]
		});
		return super.restPut(jsondata);
	}
	valueFromKey(item){
		return this.data.find(d => d['item'] === item).value;
	}
	async reflection(){
		try{
			await this.restGet(['Color style', 'reservation', 'inoutboard']);
			this.reflectionColorStyle();
			this.reflectionInoutboard();
		}catch(e){confirm(e.message);}
	}
	reflectionColorStyle(filename){
		const cssFileName = filename ? filename : this.valueFromKey('Color style');
		const cssFileUrl = `${location.origin}/ats/${cssFileName}.css`;
		this.linkColorCss.href = cssFileUrl;
	}
	async reflectionInoutboard(){
		if(this.valueFromKey('inoutboard') == 'true'){
			const spanUserName = document.querySelector('.user-name');
			spanUserName.dataset.tooltip = '出退勤 行先';
			spanUserName.classList.add('attendance');
			spanUserName.addEventListener('click', this.onUserNameClick);
			try{
				if(Object.keys(atsAttendance.data).length === 0)
					await atsAttendance.restGet(atsCurrentUserId);
			}catch(e){updateUserAttendance();}
		}else{
			const spanUserName = document.querySelector('.user-name');
			delete spanUserName.dataset.tooltip;
			spanUserName.classList.remove('attendance');
			spanUserName.removeEventListener('click', this.onUserNameClick);
			atsAttendance.data =[];
		}
		updateUserAttendance();
	}
	async onUserNameClick(){
		if(!this.loadedInoutEditLayer){
			await this.loadInoutEditLayer();
		}else{
			showInoutEditLayer(atsCurrentUserId);	// inoutboard.js
		}
	}

	async loadInoutEditLayer(){
		try{
			const inoutboardCssFileUrl = `${location.origin}/ats/inoutboard.css`;
			const inoutboardHtmlFileUrl = `${location.origin}/ats/inoutboard.html`;
			const inoutboardJsFileUrl = `${location.origin}/ats/inoutboard.js`;

			const linkCssInoutboard = document.createElement('link');
			linkCssInoutboard.rel = 'stylesheet';
			linkCssInoutboard.type = 'text/css';
			linkCssInoutboard.href = inoutboardCssFileUrl;
			document.head.appendChild(linkCssInoutboard);

			await fetch(inoutboardHtmlFileUrl)
				.then(res => res.text())
				.then(data => {
					document.body.insertAdjacentHTML('beforeend', data);
				});

			const scriptInoutboard = document.createElement('script');
			scriptInoutboard.src = inoutboardJsFileUrl;
			scriptInoutboard.defer = true;
			document.head.appendChild(scriptInoutboard);
			this.loadedInoutEditLayer = true;
		}catch(e){confirm(e.message);}
	}
}

class restUsers extends restClient {
	constructor(){
		super(`${basePath}/users`, 'users');
	}
	restGet(userArg, withDisable){
		if(typeof userArg === 'number'){
			const query = new URLSearchParams({'user_id': userArg});
			this.url = `${this.baseURL}?${query}`;
		}else if(typeof userArg === 'string'){
			const query = new URLSearchParams({'user_account': userArg});
			this.url = `${this.baseURL}?${query}`;
		}else if(userArg === undefined){
			if(withDisable === undefined){
				this.url = this.baseURL;
			}else{
				const query = new URLSearchParams({'with_disable': 1});
				this.url = `${this.baseURL}?${query}`;
			}
		}
		return super.restGet()
	}
	restPost(userAccount, userName, userDesc, userSeries){
		if(!userAccount || !userName || (userDesc===undefined) || (userSeries===undefined)){
			throw Error(`${this.constructor.name}.restPost: all required argument.`);
		}
		this.url = this.baseURL;
		const jsondata = JSON.stringify({
			[this.jsonRoot]:[{
				'user_account': userAccount,
				'user_name': userName,
				'user_desc': userDesc,
				'user_series': userSeries
			}]
		});
		return super.restPost(jsondata);
	}
	restPut(userId, userAccount, userName, userDesc, userSeries, withDisable){
		if(!userId){throw Error(`${this.constructor.name}.restPut: all required argument.`);}
		const source = this.findField('user_id', userId);
		const dest = {};
		if(source){
			dest.user_id = source.user_id;
			dest.user_account = userAccount ? userAccount : source.user_code;
			dest.user_name = userName ? userName : source.user_name;
			dest.user_desc = altHCO(userDesc, source.user_desc);
			dest.user_series = altHCO(userSeries, source.user_series);
			dest.disable = withDisable;
		}else{
			if(!userId || !userAccount || !userName || (userDesc===undefined) || (userSeries===undefined)){
				throw Error(`${this.constructor.name}.restPut: all required argument.`);
			}else{
				dest.user_id = userId;
				dest.user_account = userAccount;
				dest.user_name = userName;
				dest.user_desc = userDesc;
				dest.user_series = userSeries;
				dest.user_disable = withDisable;
			}
		}
		const query = new URLSearchParams({'user_id': userId});
		this.url = `${this.baseURL}?${query}`;
		const jsondata = JSON.stringify({[this.jsonRoot]:[dest]});
		return super.restPut(jsondata);
	}
	restDelete(userId){
		if(!userId){throw Error(`${this.constructor.name}.restDelete: userId is a required argument.`);}
		const query = new URLSearchParams({
			'user_id': userId,
		});
		this.url = `${this.baseURL}?${query}`;
		return super.restDelete();
	};
	restOrderPut(userIds){
		if(!userIds){throw Error(`${this.constructor.name}.restOrderPut: userIds is a required argument.`);}
		this.url = `${this.baseURL}/order`;
		const jsondata = JSON.stringify({[this.jsonRoot]:userIds});
		return super.restPut(jsondata);
	}
}

class restReservationTables extends restClient {
	constructor(){
		super(`${basePath}/tables`, 'tables');
	};
	restGet(tableId, withDisable){
		if(tableId === undefined){
			if(withDisable === undefined){
				this.url = this.baseURL;
			}else{
				const query = new URLSearchParams({'with_disable': 1});
				this.url = `${this.baseURL}?${query}`;
			}
		}else{
			const query = new URLSearchParams({'table_id': tableId});
			this.url = `${this.baseURL}?${query}`;
		}
		return super.restGet()
	}
	restPost(tableName, tableCustom, tableUri, tableDesc, itemName, itemBulk){
		if(!tableName || (tableCustom===undefined) || !tableUri  || (tableDesc===undefined) || (itemName===undefined) || (itemBulk===undefined)){
			throw Error(`${this.constructor.name}.restPost: all required argument.`);
		}
		this.url = this.baseURL;
		const jsondata = JSON.stringify({
			[this.jsonRoot]:[{
				'table_name': tableName,
				'table_custom': tableCustom,
				'table_uri': tableUri,
				'table_desc': tableDesc,
				'item_name': itemName,
				'item_bulk': itemBulk
			}]
		});
		return super.restPost(jsondata);
	}
	restPut(tableId, tableName, tableCustom, tableUri, tableDesc, itemName, itemBulk, withDisable){
		if(!tableId){throw Error(`${this.constructor.name}.restPut: tableId is a required argument.`);}
		const source = this.findField('table_id', tableId);
		const dest = {};
		if(source){
			dest.table_id = source.table_id;
			dest.table_name = tableName ? tableName : source.table_name;
			dest.table_custom = (tableCustom != undefined) ? tableCustom : source.table_custom;
			dest.table_uri = tableUri ? tableUri : source.table_uri;
			dest.table_desc = altHCO(tableDesc, source.table_desc);
			dest.item_name = altHCO(itemName, source.item_name);
			dest.item_bulk = altHCO(itemBulk, source.item_bulk);
			dest.disable = (withDisable != undefined) ? withDisable : source.disable;
		}else{
			if(!tableId || !tableName || (tableCustom===undefined) || !tableUri  || (tableDesc===undefined) || (itemName===undefined) || (itemBulk===undefined)){
				throw Error(`${this.constructor.name}.restPut: required argument.`);
			}else{
				dest.table_id = tableId;
				dest.table_name = tableName;
				dest.table_custom = tableCustom;
				dest.table_uri = tableUri;
				dest.table_desc = tableDesc;
				dest.item_name = itemName;
				dest.item_bulk = itemBulk;
				dest.disable = withDisable;
			}
		}
		this.url = `${this.baseURL}`;
		const jsondata = JSON.stringify({[this.jsonRoot]:[dest]});
		return super.restPut(jsondata);
	}
	restDelete(tableId){
		if(!tableId){throw Error(`${this.constructor.name}.restDelete: tableId is a required argument.`);}
		const query = new URLSearchParams({
			'table_id': tableId,
		});
		this.url = `${this.baseURL}?${query}`;
		return super.restDelete();
	};
	restOrderPut(tableIds){
		if(!tableIds){throw Error(`${this.constructor.name}.restOrderPut: tableIds is a required argument.`);}
		this.url = `${this.baseURL}/order`;
		const jsondata = JSON.stringify({[this.jsonRoot]:tableIds});
		return super.restPut(jsondata);
	}
}

class restItems extends restClient {
	constructor(){
		super(`${basePath}/items`, 'items');
		this.tableId = 0;
	};
	restGet(tableId, withDisable){
		this.tableId = tableId ? tableId : this.tableId;
		if(!this.tableId){throw Error(`${this.constructor.name}.restGet: tableId is not set.`);}
		const query = new URLSearchParams({'table_id': this.tableId});
		if(withDisable != undefined){query.append('with_disable',1);}
		this.url = `${this.baseURL}?${query}`;
		return super.restGet();
	}
	restPost(itemName, itemDesc){
		if(!this.tableId){throw Error(`${this.constructor.name}.restPost: tableId is not set.`);}
		if(!itemName || (itemDesc===undefined)){
			throw Error(`${this.constructor.name}.restPost: all required argument.`);
		}
		const query = new URLSearchParams({'table_id': this.tableId});
		this.url = `${this.baseURL}?${query}`;
		const jsondata = JSON.stringify({
			[this.jsonRoot]:[{
				'item_name': itemName,
				'item_desc': itemDesc
			}]
		});
		return super.restPost(jsondata);
	}
	restPut(itemId, itemName, itemDesc, withDisable){
		if(!itemId){throw Error(`${this.constructor.name}.restPut: itemId is a required argument.`);}
		const source = this.findField('item_id', itemId);
		const dest = {};
		if(source){
			dest.item_id = source.item_id;
			dest.item_name = itemName ? itemName : source.item_name;
			dest.item_desc = altHCO(itemDesc, source.item_desc);
			dest.disable = withDisable;
		}else{
			if(!itemId || !itemName || !itemDesc){
				throw Error(`${this.constructor.name}.restPut: required argument.`);
			}else{
				dest.item_id = itemId;
				dest.item_name = itemName;
				dest.item_desc = itemDesc;
				dest.disable = withDisable;
			}
		}
		const query = new URLSearchParams({'table_id': this.tableId});
		this.url = `${this.baseURL}?${query}`;
		const jsondata = JSON.stringify({[this.jsonRoot]:[dest]});
		return super.restPut(jsondata);
	}
	restDelete(itemId){
		if(!this.tableId){throw Error(`${this.constructor.name}.restDelete: tableId is not set.`);}
		if(!itemId){throw Error(`${this.constructor.name}.restDelete: itemId is a required argument.`);}
		const query = new URLSearchParams({
			'table_id': this.tableId,
			'item_id': itemId
		});
		this.url = `${this.baseURL}?${query}`;
		return super.restDelete();
	};
	restOrderPut(itemIds){
		if(!itemIds){throw Error(`${this.constructor.name}.restOrderPut: itemIds is a required argument.`);}
		this.url = `${this.baseURL}/order`;
		const jsondata = JSON.stringify({[this.jsonRoot]:itemIds});
		return super.restPut(jsondata);
	}
}

class restCalendar extends restClient {
	constructor(){
		super(`${basePath}/calendar`, 'calendar');
		this.startDate = new Date();
		this.endDate = new Date();
	}
	restGet(startDate, endDate){
		this.startDate.setTime(
			startDate ? startDate.getTime() : this.startDate.getTime()
		);
		this.endDate.setTime(
			endDate ? endDate.getTime() : this.endDate.getTime()
		);
		const query = new URLSearchParams({
			'start_date': formatDate(this.startDate, 'yyyy-MM-dd'),
			'end_date': formatDate(this.endDate, 'yyyy-MM-dd')
		});
		this.url = `${this.baseURL}?${query}`;
		return super.restGet();
	}
	restPut(startDate, endDate, holData){
		this.startDate.setTime(
			startDate ? startDate.getTime() : this.startDate.getTime()
		);
		this.endDate.setTime(
			endDate ? endDate.getTime() : this.endDate.getTime()
		);
		this.data = holData ? holData : this.data;
		const query = new URLSearchParams({
			'start_date': formatDate(this.startDate, 'yyyy-MM-dd'),
			'end_date': formatDate(this.endDate, 'yyyy-MM-dd')
		});
		this.url = `${this.baseURL}?${query}`;
		const jsondata = JSON.stringify({[this.jsonRoot]:this.data});
		return super.restPut(jsondata);
	}
	dateType(targetDate){
		// return = undefined(平日扱) || lh(legal holiday:法定休日) || sh(schedule holiday:所定休日)
		const dateData = this.findField('date', formatDate(targetDate, 'yyyy-MM-dd'));
		if(dateData !== undefined){
			// ユーザー定義がある場合、所定休日のみ反映
			if(dateData.type === 'sh'){return 'sh';}
		}else{
			// ユーザー定義が無い場合、日曜を法定休日、土曜を所定休日とする
			// 曜日は仮処置。いずれはこの部分もユーザー定義出来るようにする。
			if(targetDate.getDay()===0){return 'lh';}
			if(targetDate.getDay()===6){return 'sh';}
		}
	}
}

class restReservations extends restClient {
	constructor(){
		super(`${basePath}/reservations`, 'reservations');
		this.tableId = 0;
		this.startDateTime = new Date();
		this.endDateTime = new Date();
	}
	restGet(tableId, startDateTime, endDateTime){
		this.tableId = tableId ? tableId : this.tableId;
		this.startDateTime.setTime(
			startDateTime ? startDateTime.getTime() : this.startDateTime.getTime()
		);
		this.endDateTime.setTime(
			endDateTime ? endDateTime.getTime() : this.endDateTime.getTime()
		);
		if(!this.tableId){throw Error(`${this.constructor.name}.restGet: tableId is not set.`);}
		const query = new URLSearchParams({
			'table_id': this.tableId,
			'start_datetime': this.startDateTime.toISOString(),
			'end_datetime': this.endDateTime.toISOString()
		});
		this.url = `${this.baseURL}?${query}`;
		return super.restGet();
	}
	restGetLastUses(tableId, dateTime){
		this.tableId = tableId ? tableId : this.tableId;
		this.endDateTime.setTime(
			dateTime ? dateTime.getTime() : this.endDateTime.getTime()
		);
		if(!this.tableId){throw Error(`${this.constructor.name}.restGetLastUses: tableId is not set.`);}
		const query = new URLSearchParams({
			'table_id': this.tableId,
			'datetime': this.endDateTime.toISOString(),
			'last_uses': 1
		});
		this.url = `${this.baseURL}?${query}`;
		return super.restGet();
	}
	restPost(itemIds, userId, startDateTime, endDateTime, desc){
		if(!this.tableId){throw Error(`${this.constructor.name}.restPost: tableId is not set.`);}
		if(!itemIds || !userId || !startDateTime || !endDateTime || !desc){
			throw Error(`${this.constructor.name}.restPost: all required argument.`);
		}
		const query = new URLSearchParams({'table_id': this.tableId});
		this.url = `${this.baseURL}?${query}`;
		const jsondata = JSON.stringify({
			[this.jsonRoot]:[{
				'item_ids': itemIds,
				'user_id': userId,
				'start_datetime': startDateTime.toISOString(),
				'end_datetime': endDateTime.toISOString(),
				'reservation_desc': desc
			}]
		});
		return super.restPost(jsondata);
	}
	restPut(reservationId, itemIds, userId, startDateTime, endDateTime, desc){
		if(!reservationId){throw Error(`${this.constructor.name}.restPut: reservationId is a required argument.`);}
		const source = this.findField('reservation_id', reservationId);
		const dest = {};
		if(source){
			dest.reservation_id = source.reservation_id;
			dest.item_ids = itemIds ? itemIds : source.item_ids;
			dest.user_id = userId ? userId : source.User_id; 
			dest.start_datetime = startDateTime ? startDateTime.toISOString() : source.start_dateTime;
			dest.end_datetime = endDateTime ? endDateTime.toISOString() : source.end_dateTime;
			dest.reservation_desc = altHCO(desc, source.desc);
		}else{
			if(!itemIds || !userId || !startDateTime || !endDateTime){
				throw Error(`${this.constructor.name}.restPut: required argument.`);
			}else{
				dest.reservation_id = reservationId;
				dest.item_ids = itemIds;
				dest.user_id = userId;
				dest.start_datetime = startDateTime.toISOString();
				dest.end_datetime = endDateTime.toISOString();
				dest.reservation_desc = desc;
			}
		}
		const query = new URLSearchParams({'table_id': this.tableId});
		this.url = `${this.baseURL}?${query}`;
		const jsondata = JSON.stringify({[this.jsonRoot]:[dest]});
		return super.restPut(jsondata);
	}
	restDelete(reservationId){
		if(!this.tableId){throw Error(`${this.constructor.name}.restDelete: tableId is not set.`);}
		if(!reservationId){throw Error(`${this.constructor.name}.restDelete: reservationId is a required argument.`);}
		const query = new URLSearchParams({
			'table_id': this.tableId,
			'reservation_id': reservationId
		});
		this.url = `${this.baseURL}?${query}`;
		return super.restDelete();
	};
};

class restInoutboards extends restClient {
	constructor(){
		super(`${basePath}/inoutboards`, 'inoutboards');
	};
	restGet(withDisable){
		if(withDisable === undefined){
			this.url = this.baseURL;
		}else{
			const query = new URLSearchParams({'with_disable': 1});
			this.url = `${this.baseURL}?${query}`;
		}
		return super.restGet()
	}
	restPost(boardName){
		if(!boardName){throw Error(`${this.constructor.name}.restPost: boardName required argument.`);}
		this.url = this.baseURL;
		const jsondata = JSON.stringify({
			[this.jsonRoot]:[{
				'board_name': boardName,
			}]
		});
		return super.restPost(jsondata);
	}
	restPut(boardId, boardName, withDisable){
		if(!boardId){throw Error(`${this.constructor.name}.restPut: boardId is a required argument.`);}
		const source = this.findField('board_id', boardId);
		const dest = {};
		if(source){
			dest.board_id = source.board_id;
			dest.board_name = boardName ? boardName : source.board_name;
			dest.disable = (withDisable != undefined) ? withDisable : source.disable;
		}else{
			if(!boardId || !boardName){
				throw Error(`${this.constructor.name}.restPut: required argument.`);
			}else{
				dest.board_id = boardId;
				dest.board_name = boardName;
				dest.disable = withDisable;
			}
		}
		this.url = `${this.baseURL}`;
		const jsondata = JSON.stringify({[this.jsonRoot]:[dest]});
		return super.restPut(jsondata);
	}
	restDelete(boardId){
		if(!boardId){throw Error(`${this.constructor.name}.restDelete: boardId is a required argument.`);}
		const query = new URLSearchParams({
			'board_id': boardId,
		});
		this.url = `${this.baseURL}?${query}`;
		return super.restDelete();
	};
	restOrderPut(boardIds){
		if(!boardIds){throw Error(`${this.constructor.name}.restOrderPut: boardIds is a required argument.`);}
		this.url = `${this.baseURL}/order`;
		const jsondata = JSON.stringify({[this.jsonRoot]:boardIds});
		return super.restPut(jsondata);
	}
}

class restWhereabouts extends restClient {
	constructor(){
		super(`${basePath}/whereabouts`, 'whereabouts');
	};
	restGet(withDisable){
		if(withDisable === undefined){
			this.url = this.baseURL;
		}else{
			const query = new URLSearchParams({'with_disable': 1});
			this.url = `${this.baseURL}?${query}`;
		}
		return super.restGet()
	}
	restPost(whereabouts){
		if(!whereabouts){throw Error(`${this.constructor.name}.restPost: whereabouts required argument.`);}
		this.url = this.baseURL;
		const jsondata = JSON.stringify({
			[this.jsonRoot]:[{
				'whereabouts': whereabouts,
			}]
		});
		return super.restPost(jsondata);
	}
	restPut(whereaboutsId, whereabouts, withDisable){
		if(!whereaboutsId){throw Error(`${this.constructor.name}.restPut: whereaboutsId is a required argument.`);}
		const source = this.findField('whereabouts_id', whereaboutsId);
		const dest = {};
		if(source){
			dest.whereabouts_id = source.whereabouts_id;
			dest.whereabouts = whereabouts ? whereabouts : source.whereabouts;
			dest.disable = (withDisable != undefined) ? withDisable : source.disable;
		}else{
			if(!whereaboutsId || !whereabouts){
				throw Error(`${this.constructor.name}.restPut: required argument.`);
			}else{
				dest.whereabouts_id = whereaboutsId;
				dest.whereabouts = whereabouts;
				dest.disable = withDisable;
			}
		}
		this.url = `${this.baseURL}`;
		const jsondata = JSON.stringify({[this.jsonRoot]:[dest]});
		return super.restPut(jsondata);
	}
	restDelete(whereaboutsId){
		if(!whereaboutsId){throw Error(`${this.constructor.name}.restDelete: whereaboutsId is a required argument.`);}
		const query = new URLSearchParams({
			'whereabouts_id': whereaboutsId,
		});
		this.url = `${this.baseURL}?${query}`;
		return super.restDelete();
	};
	restOrderPut(whereaboutsIds){
		if(!whereaboutsIds){throw Error(`${this.constructor.name}.restOrderPut: whereaboutsIds is a required argument.`);}
		this.url = `${this.baseURL}/order`;
		const jsondata = JSON.stringify({[this.jsonRoot]:whereaboutsIds});
		return super.restPut(jsondata);
	}
}

class restAttendance extends restClient {
	constructor(){
		super(`${basePath}/attendance`, 'attendance');
	}
	restGet(userId, withDisable){
		if(userId === undefined){
			if(withDisable === undefined){
				this.url = this.baseURL;
			}else{
				const query = new URLSearchParams({'with_disable': 1});
				this.url = `${this.baseURL}?${query}`;
			}
		}else{
			const query = new URLSearchParams({'user_id': userId});
			this.url = `${this.baseURL}?${query}`;
		}
		return super.restGet()
	}
	restPost(userIds, boardId){
		if(!userIds || !boardId){
			throw Error(`${this.constructor.name}.restPost: all required argument.`);
		}
		this.url = this.baseURL;
		var arrayUsers = [];
		for(const userId of userIds){
			arrayUsers.push({'user_id': userId, 'board_id': boardId});
		}
		const jsondata = JSON.stringify({[this.jsonRoot]:arrayUsers});
		return super.restPost(jsondata);
	}
	restPut(userId, clockStatus, whereabouts, description){
		if(!userId){throw Error(`${this.constructor.name}.restPut: all required argument.`);}
		const source = this.findField('user_id', userId);
		const dest = {};
		if(source){
			dest.user_id = source.user_id;
			dest.clock_status = altHCO(clockStatus, source.clock_status);
			dest.whereabouts = altHCO(whereabouts, source.whereabouts);
			dest.description = altHCO(description, source.description);
		}else{
			if(!userId || !clockStatus || (whereabouts===undefined) || (description===undefined)){
				throw Error(`${this.constructor.name}.restPut: all required argument.`);
			}else{
				dest.user_id = userId;
				dest.clock_status = clockStatus;
				dest.whereabouts = whereabouts;
				dest.description = description;
			}
		}
		this.url = `${this.baseURL}`;
		const jsondata = JSON.stringify({[this.jsonRoot]:[dest]});
		return super.restPut(jsondata);
	}
	restDelete(userId){
		if(!userId){throw Error(`${this.constructor.name}.restDelete: userId is a required argument.`);}
		const query = new URLSearchParams({
			'user_id': userId,
		});
		this.url = `${this.baseURL}?${query}`;
		return super.restDelete();
	};
}
function updateUserAttendance(){
	const userName = document.querySelector('.user-name');
	const atsUser = atsAttendance.findField('user_id', atsCurrentUserId);
	if(atsUser){
		switch(atsUser.clock_status){
			case 0:
				userName.classList.add('clock-out');
				userName.classList.remove('clock-in');
				userName.classList.remove('default');
				break;
			case 1:
				userName.classList.add('clock-in');
				userName.classList.remove('clock-out');
				userName.classList.remove('default');
				break;
		}
	}else{
		userName.classList.add('default');
		userName.classList.remove('clock-in');
		userName.classList.remove('clock-out');
	}
}

const TABLE_ID_ALL = undefined;
const USER_ID_ALL = undefined;
const WITHDISABLED = 1;
const atsConfiguration = new restConfiguration();
const atsUsersTable = new restUsers();
const atsReservationTables = new restReservationTables();
const atsItems = new restItems();
const atsReservations = new restReservations();
const atsCalendar = new restCalendar();
const atsWhereabouts = new restWhereabouts();
const atsInoutboards = new restInoutboards();
const atsAttendance = new restAttendance();
